--[[
	NEED TO DO
	allow selecting to use udp or tcp
	add an option to provide servers IP address, no discovering is needed
]]

--[[
	HELPING FUNCTIONS
]]--

local function getIP()
	local s = socket.udp()
	s:setpeername("74.125.115.104",80)
	local ip, _ = s:getsockname()
	return ip
end

local function explode(div,str) -- credit: http://richard.warburton.it
	if (div=='') then return false end
	local pos,arr = 0,{}
	-- for each divider found
	for st,sp in function() return string.find(str,div,pos,true) end do
		table.insert(arr,string.sub(str,pos,st-1)) -- Attach chars left of current divider
		pos = sp + 1 -- Jump past current divider
	end
	table.insert(arr,string.sub(str,pos)) -- Attach chars right of last divider
	return arr
end

--[[
	CLIENT STUFF
]]--

Client = Core.class(EventDispatcher)

function Client:init(config)
	self.conf = {
		port = 15000,
		multicast = "239.192.1.1",
		serverIP = nil,
		connectionType = "tcp"
	}
	if config then
		--copying configuration
		for key,value in pairs(config) do
			self.conf[key] = value
		end
	end
	
	--get IP address of active interface
	self.ip = getIP()
	local host = socket.dns.tohostname(self.ip)
	self.host = host or self.ip
	self.port = self.conf.port
	
	--store added methods
	self.methods = {}
	
	--store available server
	self.servers = {}
	
	--generate random id
	math.randomseed(os.time())
	self.id = tostring(math.random(os.time()))
	
	--is connected to server
	self.connected = false
	
	--create timer for listening
	self.listenTimer = Timer.new(100)
    --setting timer callback
    self.listenTimer:addEventListener(Event.TIMER, Client.ListenStep, self)
	
	self.newServerEvent = Event.new("newServer")
	self.acceptedEvent = Event.new("onAccepted")
	self.connectionClosedEvent = Event.new("onServerClose")
	self.clientClosedEvent = Event.new("onClientClose")
end

function Client:startListening()
	if not self.connected then
		--create udp instance
		self.udp = socket.udp()
	
		--set socket for multicast
		self.udp:setsockname(self.conf.multicast, self.port)
	
		--check if supports multicast
		local name = self.udp:getsockname()
		if(name)then
			--add interface to multicast
			self.udp:setoption("ip-add-membership" , { multiaddr = self.conf.multicast, interface = self.ip})
		else
			--create udp instance
			self.udp = socket.udp()
			--does not support multicast, but will probably receive broadcast message
			self.udp:setsockname(self.ip, self.port)
		end
	
		--set timeout so it won't block UI
		self.udp:settimeout(0)
	end
	self.listenTimer:start()
end

function Client:ListenStep()
	if(not self.connected)then
		repeat
			--get all data
			local data, ip, port = self.udp:receivefrom()
			--if there is any data
			if data then
				--get parameters
				local id, cmd, params = data:match("^(%S*) (%S*) (.*)")
				--check if sender not itself
				if self.id ~= id and cmd == 'unite' then
					if self.servers[id] == nil then
						local h = explode(" ", params)
						local host = h[1]
						self.servers[id] = {}
						self.servers[id].ip = ip
						self.servers[id].host = host
						--add new server
						self.newServerEvent.data = {id = id, ip = ip, host = host}
						self:dispatchEvent(self.newServerEvent)
					end
				end
			end
		until not data
	end
	if self.tcp then
		repeat
			local data, err = self.tcp:receive()
			if data then 
				local id, cmd, params = data:match("^(%S*) (%S*) (.*)")
				if self.id ~= id then
					if cmd == 'accept' then
						--server accepted us, so we are connected
						self.connected = true
						if self.udp then
							self.udp:close()
							self.udp = nil
						end
						self.servers = {}
						self:dispatchEvent(self.acceptedEvent)
					elseif cmd == 'closed' then
						local p = explode(" ", params)
						self.clientClosedEvent.data = {id = p[1]}
						self:dispatchEvent(self.clientClosedEvent)
					elseif(self.methods[cmd]) then
						local p = explode(" ", params)
						if p then
							p[#p] = id
							if self.methods[cmd].scope then
								self.methods[cmd].method(self.methods[cmd].scope, unpack(p))
							else
								self.methods[cmd].method(unpack(p))
							end
						else
							if self.methods[cmd].scope then
								self.methods[cmd].method(self.methods[cmd].scope, id)
							else
								self.methods[cmd].method(id)
							end
						end
					end
				end
			end
		until not data
	end
end

function Client:stopListening()
	self.listenTimer:stop()
	if self.udp then
		self.udp:close()
		self.udp = nil
	end
end

function Client:connect(id)
	if self.servers[id] then
		--connect to server
		self.tcp = socket.connect(self.servers[id].ip, self.port)
		--set timeout so it won't block UI
		self.tcp:settimeout(0)
		local msg = string.format("%s %s %s %s", self.id, 'join', self.host, "\n")
		self:sendMsg(msg)
	else
		error("Error: There is no server with id: "..id, 2)
	end
end

function Client:sendMsg(msg)
	local byte, err = self.tcp:send(msg)
	if err == "closed" then
		self:dispatchEvent(self.connectionClosedEvent)
	end
end

function Client:addMethod(name, method, scope)
	self.methods[name] = {}
	self.methods[name].method = method
	self.methods[name].scope = scope
end

function Client:callMethod(...)
	local msg = self.id.." "..table.concat(arg, " ").." \n"
	self:sendMsg(msg)
end

function Client:callMethodOf(...)
	local id = arg[#arg]
	arg[#arg] = nil
	local msg = self.id.." proxy "..id.." "..table.concat(arg, " ").." \n"
	self:sendMsg(msg)
end

function Client:close()
	self:stopListening()
	if self.tcp then
		self.tcp:close()
		self.tcp = nil
	end
	self = nil
end

--[[
	SERVER STUFF
]]--

Server = Core.class(EventDispatcher)

function Server:init(config)
	self.conf = {
		port = 15000,
		multicast = "239.192.1.1",
		maxClients = 0
	}
	if config then
		--copying configuration
		for key,value in pairs(config) do
			self.conf[key] = value
		end
	end
	
	--get IP address of active interface
	self.ip = getIP()
	self.port = self.conf.port
	local host = socket.dns.tohostname(self.ip)
	self.host = host or self.ip
	
	--store added methods
	self.methods = {}
	
	--store clients
	self.clients = {}
	self.clientsMap = {}
	self.clientsIds = {}
	
	--client count
	self.clientCount = 0
	
	--temporary client list
	self.tempClients = {}
	
	--if game is not started, still accepts new connections
	self.gameStarted = false
	
	--generate random id
	math.randomseed(os.time())
	self.id = tostring(math.random(os.time()))
	
	self.server = assert(socket.bind(self.ip, self.port))
	self.server:settimeout(0)

	--create broadcast timer
	self.broadcastTimer = Timer.new(100)
    --setting timer callback
    self.broadcastTimer:addEventListener(Event.TIMER, Server.broadcastStep, self)
	
	--create timer for listening
	self.listenTimer = Timer.new(100)
    --setting timer callback
    self.listenTimer:addEventListener(Event.TIMER, Server.ListenStep, self)
	
	--new client event
	self.newClientEvent = Event.new("newClient")
	self.connectionClosedEvent = Event.new("onClientClose")
end

function Server:startBroadcast()
	--create udp instance
	self.udp = socket.udp()
	
	--set socket for ip
	self.udp:setsockname(self.ip, self.port)
	
	self.gameStarted = false
	
	--set timeout so it won't block UI
	self.udp:settimeout(0)
	
	self.broadcastTimer:start()
end

function Server:broadcastStep()
	--create message
	local msg = string.format("%s %s %s %s", self.id, 'unite', self.host, "\n")
	
	--send to multicast group
	assert(self.udp:sendto(msg, self.conf.multicast, self.port))
	
	--enable broadcast
	self.udp:setoption('broadcast', true)
	--self.udp:setoption('dontroute', true)
	--broadcast message
	assert(self.udp:sendto(msg, "255.255.255.255", self.port))
	--disable broadcast
	self.udp:setoption('broadcast', false)
	--self.udp:setoption('dontroute', false)
	
	--listen for clients
	self:ListenStep()
end

function Server:stopBroadcast()
	self.broadcastTimer:stop()
	self.gameStarted = true
	if self.udp then
		self.udp:close()
		self.udp = nil
	end
end

function Server:addMethod(name, method, scope)
	self.methods[name] = {}
	self.methods[name].method = method
	self.methods[name].scope = scope
end

function Server:callMethod(...)
	local msg = self.id.." "..table.concat(arg, " ").." \n"
	self:sendMsg(msg)
end

function Server:callMethodOf(...)
	local id = arg[#arg]
	local to = self.clientsMap[id].id
	arg[#arg] = nil
	local msg = self.id.." "..table.concat(arg, " ").." \n"
	self:sendMsgTo(msg, to)
end

function Server:sendMsg(msg, exclude)
	local sender
	if self.clientsMap[exclude] then
		sender = self.clientsMap[exclude].id
	end
	for i = 1, #self.clients do
		if sender ~= i then
			self:sendMsgTo(msg, i)
		end
	end
end

function Server:sendMsgTo(msg, to)
	local byte, err = self.clients[to]:send(msg)
	if err == "closed" then
		local id = self.clientsIds[to]
		local client = self.clientsMap[id]
		if client.connected then
			client.connected = false
			self.newClientEvent.data = {id = client.id, ip = client.ip, host = client.host}
			self:dispatchEvent(self.connectionClosedEvent)
			local msg = string.format("%s %s %s %s", self.id, 'closed', id, "\n")
			self:sendMsg(msg, id)
		end
	end
end

function Server:startListening()
	self.listenTimer:start()
end

function Server:ListenStep()
	if not self.gameStarted then
		repeat
			--if maximal clients set and reached that limit
			if self.conf.maxClients > 0 and self.clientCount >= self.conf.maxClients then
				--do not accept any new connections
				break
			end
			local client = self.server:accept()
			if client then
				client:settimeout(0)
				self.tempClients[#self.tempClients+1] = client
			end
		until not client
		local ready, none, err = socket.select(self.tempClients, self.tempClients, 0)
		if err == nil then
			for i = 1, #ready do
				local client = ready[i]
				local data, err = client:receive()
				if data then 
					local id, cmd, params = data:match("^(%S*) (%S*) (.*)")
					if cmd == 'join' and self.clientsMap[id] == nil then
						local h = explode(" ", params)
						--get info about client
						local ip = client:getpeername()
						local host = h[1]
						--someone wants to join server
						local clientID = #self.clients + 1
						--store client's ID
						self.clientsMap[id] = {}
						self.clientsMap[id].id = clientID
						self.clientsMap[id].ip = ip
						self.clientsMap[id].host = host
						self.clientsMap[id].connected = true
						self.clientsIds[clientID] = id
						--store client
						self.clients[clientID] = client
						--add new server
						self.newClientEvent.data = {id = id, ip = ip, host = host}
						self:dispatchEvent(self.newClientEvent)
					end
				end
			end
		end
	end
	
	local ready, none, err = socket.select(self.clients, self.clients, 0)
	if ready then
		for i = 1, #ready do
			local client = ready[i]
			repeat
				-- receive the line
				local data, err = client:receive()
				-- if there was no error, send it back to the client
				if data then 
					local id, cmd, params = data:match("^(%S*) (%S*) (.*)")
					if cmd == "proxy" then
						local p = explode(params)
						local id = p[1]
						p[1] = nil
						p[#p] = nil
						local msg = id.." "..table.concat(p, " ").." \n"
						self:sendMsgTo(msg, self.clientMaps[id].id)
					elseif(self.methods[cmd]) then
						local p = explode(" ", params)
						if p then
							p[#p] = id
							if self.methods[cmd].scope then
								self.methods[cmd].method(self.methods[cmd].scope, unpack(p))
							else
								self.methods[cmd].method(unpack(p))
							end
						else
							if self.methods[cmd].scope then
								self.methods[cmd].method(self.methods[cmd].scope, id)
							else
								self.methods[cmd].method(id)
							end
						end
					end
					self:sendMsg(data.."\n", id)
				end
			until not data
		end
	end
end

function Server:accept(id)
	if self.clientsMap[id] then
		local client = self.clients[self.clientsMap[id].id]
		local msg = string.format("%s %s %s", self.id, 'accept', "\n")
		client:send(msg)
	else
		error("Error: There is no client with id: "..id, 2)
	end
end

function Server:stopListening()
	self.listenTimer:stop()
end

function Server:close()
	self:stopBroadcast()
	self:stopListening()
	for i = 1, #self.clients do
		self.clients[i]:close()
	end
	self.clients = nil
	self.server:close()
	self = nil
end